import 'dart:collection';

import 'package:meta/meta.dart';
import 'package:miodi/core/consts.dart';

import 'module.dart';
import 'providers.dart';
import '../error/errors.dart';

abstract class MiodiComponent {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  final Map<String, Map<Type, AsyncProvider>> factoryAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> singletonAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> customAsyncConfiguration = HashMap();

  @mustCallSuper
  MiodiComponent() {
    if (_modules == null) {
      _modules = modules;
    }
    _modules?.forEach((module) {
      module.factoryConfiguration.forEach((key, map) {
        if (factoryConfiguration.containsKey(key)) {
          factoryConfiguration[key].addAll(map);
        } else {
          factoryConfiguration[key] = map;
        }
      });
      module.singletonConfiguration.forEach((key, map) {
        if (singletonConfiguration.containsKey(key)) {
          singletonConfiguration[key].addAll(map);
        } else {
          singletonConfiguration[key] = map;
        }
      });
      module.customConfiguration.forEach((key, map) {
        if (customConfiguration.containsKey(key)) {
          customConfiguration[key].addAll(map);
        } else {
          customConfiguration[key] = map;
        }
      });

      module.factoryAsyncConfiguration.forEach((key, map) {
        if (factoryAsyncConfiguration.containsKey(key)) {
          factoryAsyncConfiguration[key].addAll(map);
        } else {
          factoryAsyncConfiguration[key] = map;
        }
      });
      module.singletonAsyncConfiguration.forEach((key, map) {
        if (singletonAsyncConfiguration.containsKey(key)) {
          singletonAsyncConfiguration[key].addAll(map);
        } else {
          singletonAsyncConfiguration[key] = map;
        }
      });
      module.customAsyncConfiguration.forEach((key, map) {
        if (customAsyncConfiguration.containsKey(key)) {
          customAsyncConfiguration[key].addAll(map);
        } else {
          customAsyncConfiguration[key] = map;
        }
      });
    });
  }

  List<MiodiModule> get modules;
  List<MiodiModule> _modules;

  T get<T>({String name = defaultProviderName}) {
    if (singletonConfiguration[name]?.containsKey(T) ?? false)
      return singletonConfiguration[name][T].provide() as T;
    if (factoryConfiguration[name]?.containsKey(T) ?? false)
      return factoryConfiguration[name][T].provide() as T;
    if (customConfiguration[name]?.containsKey(T) ?? false)
      return customConfiguration[name][T].provide() as T;
    throw NotFoundProviderException(T);
  }

  Future<T> getAsync<T>({String name = defaultProviderName}) {
    if (singletonAsyncConfiguration[name]?.containsKey(T) ?? false)
      return singletonAsyncConfiguration[name][T].provide();
    if (factoryAsyncConfiguration[name]?.containsKey(T) ?? false)
      return factoryAsyncConfiguration[name][T].provide();
    if (customAsyncConfiguration[name]?.containsKey(T) ?? false)
      return customAsyncConfiguration[name][T].provide();
    throw NotFoundProviderException(T);
  }
}

class DefaultMiodiComponent extends MiodiComponent {
  final List<MiodiModule> moduleList;
  DefaultMiodiComponent(this.moduleList);

  @override
  List<MiodiModule> get modules => moduleList;

}