import 'dart:collection';

import 'package:meta/meta.dart';

import 'providers.dart';

abstract class MiodiModule {
  final Map<String, Map<Type, Provider>> factoryConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> singletonConfiguration = HashMap();
  final Map<String, Map<Type, Provider>> customConfiguration = HashMap();

  final Map<String, Map<Type, AsyncProvider>> factoryAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> singletonAsyncConfiguration = HashMap();
  final Map<String, Map<Type, AsyncProvider>> customAsyncConfiguration = HashMap();

  @mustCallSuper
  MiodiModule() {
    if (_providers == null) {
      _providers = providers;
    }
    if (_providers != null) {
      _providers.forEach((provider) {
            _saveProvider(provider);
          });
    }
    if (_asyncProviders == null) {
      _asyncProviders = asyncProviders;
    }
    if (_asyncProviders != null) {
      _asyncProviders.forEach((provider) {
        _saveAsyncProvider(provider);
      });
    }
  }

  void _saveProvider(Provider provider) {
    final name = provider.name;
    if (provider is SingletonProvider) {
      if (!singletonConfiguration.containsKey(name))
        singletonConfiguration.addAll({name: HashMap()});
      _addProviderToConfiguration(provider, singletonConfiguration[name]);
    } else if (provider is FactoryProvider) {
      if (!factoryConfiguration.containsKey(name))
        factoryConfiguration.addAll({name: HashMap()});
      _addProviderToConfiguration(provider, factoryConfiguration[name]);
    } else {
      if (!customConfiguration.containsKey(name))
        customConfiguration.addAll({name: HashMap()});
      _addProviderToConfiguration(provider, customConfiguration[name]);
    }
  }

  void _saveAsyncProvider(AsyncProvider provider) {
    final name = provider.name;
    if (provider is SingletonAsyncProvider) {
      if (!singletonAsyncConfiguration.containsKey(name))
        singletonAsyncConfiguration.addAll({name: HashMap()});
      _addAsyncProviderToConfiguration(provider, singletonAsyncConfiguration[name]);
    } else if (provider is FactoryAsyncProvider) {
      if (!factoryAsyncConfiguration.containsKey(name))
        factoryAsyncConfiguration.addAll({name: HashMap()});
      _addAsyncProviderToConfiguration(provider, factoryAsyncConfiguration[name]);
    } else {
      if (!customAsyncConfiguration.containsKey(name))
        customAsyncConfiguration.addAll({name: HashMap()});
      _addAsyncProviderToConfiguration(provider, customAsyncConfiguration[name]);
    }
  }

  void _addProviderToConfiguration(
      Provider provider, Map<Type, Provider> configuration) {
    if (configuration.containsKey(provider.provideType)) {
      configuration[provider.provideType] = provider;
    } else
      configuration.addAll({provider.provideType: provider});
  }

  void _addAsyncProviderToConfiguration(
      AsyncProvider provider, Map<Type, AsyncProvider> configuration) {
    if (configuration.containsKey(provider.provideType)) {
      configuration[provider.provideType] = provider;
    } else
      configuration.addAll({provider.provideType: provider});
  }
  List<Provider> get providers;
  List<Provider> _providers;
  List<AsyncProvider> get asyncProviders;
  List<AsyncProvider> _asyncProviders;
}

class DefaultMiodiModule extends MiodiModule{
  final List<Provider> providersList;
  final List<AsyncProvider> asyncProvidersList;

  DefaultMiodiModule({this.providersList, this.asyncProvidersList});

  @override
  List<AsyncProvider> get asyncProviders => asyncProvidersList;

  @override
  List<Provider> get providers => providersList;
}