library miodi;

import 'dart:async';
import 'core/consts.dart';

import 'core/component.dart';
import 'core/configuration.dart';
import 'core/module.dart';
import 'core/providers.dart';
import 'error/errors.dart';

export 'core/component.dart';
export 'core/module.dart';
export 'core/providers.dart';

Provider<T> singleton<T>(Type T, Creator<T> creator,
        {String name = defaultProviderName, bool initOnStart = false}) =>
    SingletonProvider(T, creator, name: name, initOnStart: initOnStart);

Provider<T> factory<T>(Type T, Creator<T> creator,
        {String name = defaultProviderName}) =>
    FactoryProvider(T, creator, name: name);

AsyncProvider<T> asyncSingleton<T>(Type T, AsyncCreator<T> creator,
        {String name = defaultProviderName, bool initOnStart = false}) =>
    SingletonAsyncProvider(T, creator, name: name, initOnStart: initOnStart);

AsyncProvider<T> asyncFactory<T>(Type T, AsyncCreator<T> creator,
        {String name = defaultProviderName}) =>
    FactoryAsyncProvider(T, creator, name: name);

MiodiModule module(
        {List<Provider> providers, List<AsyncProvider> asyncProviders}) =>
    DefaultMiodiModule(providersList: providers, asyncProvidersList: asyncProviders);

MiodiComponent component(List<MiodiModule> modules) => DefaultMiodiComponent(modules);

T inject<T>([String name]) => Miodi.inject<T>(name);

Future<T> injectAsync<T>([String name]) => Miodi.injectAsync<T>(name);

class Miodi {
  final MiodiConfiguration _configuration;
  static Miodi _instance;

  Miodi._(this._configuration);

  static void init(List<MiodiComponent> components) {
    if (_instance == null)
      _instance = Miodi._(MiodiConfiguration(components));
    else
      addComponents(components);
  }

  static void addComponents(List<MiodiComponent> components) {
    if (_instance == null) throw NotInitializeMiodiException();
    _instance._configuration.addComponents(components);
  }

  static T inject<T>([String name]) {
    if (_instance == null) throw NotInitializeMiodiException();
    if (name == null) {
      return _instance._configuration.provide<T>();
    } else
      return _instance._configuration.provide<T>(name: name);
  }

  static Future<T> injectAsync<T>([String name]) async {
    if (_instance == null) throw NotInitializeMiodiException();
    if (name == null) {
      return _instance._configuration.provideAsync<T>();
    } else
      return _instance._configuration.provideAsync<T>(name: name);
  }
}
