import 'package:miodi/miodi.dart';
import 'package:test/test.dart';

void main() {
  Miodi.init([appComponent]);

  test('Test factory providers', () {
    User user1 = Miodi.inject();
    expect(userCounter, equals(1));
    User user2 = Miodi.inject('provider_name');
    expect(userCounter, equals(2));
    User user3 = Miodi.inject();
    expect(userCounter, equals(3));
  });

  test('Test singleton providers', () {
    Database database1 = Miodi.inject();
    expect(databaseCounter, 1);
    Database database2 = Miodi.inject();
    expect(databaseCounter, 1);
    Database database3 = Miodi.inject();
    expect(databaseCounter, 1);
  });

  test('Test async inject', () async {
    User user = await Miodi.injectAsync<User>('async');
    expect(user, isNotNull);
  });

//  test('Test dependency inject', () {
//    var testDependency = TestDependency();
//    expect(testDependency.database, isNotNull);
//    expect(testDependency.user, isNotNull);
//  });

  return;
}

final appComponent = AppComponent();

//class TestDependency {
//  Database database;
//  User _user;
//
//  TestDependency() {
//    Miodi.injectDependency(this, [
//      dependency(type: Database, memberName: #database),
//      dependency(type: User, memberName: #_user)
//    ]);
//  }
//
//  User get user => _user;
//}

class AppComponent extends MiodiComponent {
  @override
  List<MiodiModule> get modules => [
    module(providers: [
      singleton(Database, () => DatabaseImpl(), initOnStart: true),
      factory(User, () => User()),
      factory(User, () => User(), name: 'provider_name'),
    ], asyncProviders: [
      asyncSingleton(User, () async {
        await Future.delayed(Duration(seconds: 1));
        return User();
      }, name: 'async', initOnStart: true)
    ])
  ];
}

var databaseCounter = 0;
abstract class Database {
  void printDatabaseCounter();
}

class DatabaseImpl implements Database {
  DatabaseImpl() {
    databaseCounter++;
    printDatabaseCounter();
  }

  @override
  void printDatabaseCounter() {
    print("Database: " + databaseCounter.toString());
  }
}

var userCounter = 0;
class User {
  User() {
    userCounter++;
    print('User: $userCounter');
  }
}

